@@ -6,7 +6,7 @@ struct Diagonal{T,V<:AbstractVector{T}} <: AbstractMatrix{T}
     diag::V
 
     function Diagonal{T,V}(diag) where {T,V<:AbstractVector{T}}
-        @assert !has_offset_axes(diag)
+        require_one_based_indexing(diag)
         new{T,V}(diag)
     end
 end
@@ -171,13 +171,13 @@ end
     lmul!(D, copyto!(similar(A, promote_op(*, eltype(A), eltype(D.diag)), size(A)), A))
 
 function rmul!(A::AbstractMatrix, D::Diagonal)
-    @assert !has_offset_axes(A)
+    require_one_based_indexing(A)
     A .= A .* permutedims(D.diag)
     return A
 end
 
 function lmul!(D::Diagonal, B::AbstractMatrix)
-    @assert !has_offset_axes(B)
+    require_one_based_indexing(B)
     B .= D.diag .* B
     return B
 end
@@ -319,7 +319,7 @@ function ldiv!(D::Diagonal{T}, v::AbstractVector{T}) where {T}
     v
 end
 function ldiv!(D::Diagonal{T}, V::AbstractMatrix{T}) where {T}
-    @assert !has_offset_axes(V)
+    require_one_based_indexing(V)
     if size(V,1) != length(D.diag)
         throw(DimensionMismatch("diagonal matrix is $(length(D.diag)) by $(length(D.diag)) but right hand side has $(size(V,1)) rows"))
     end
@@ -347,7 +347,7 @@ function ldiv!(D::Diagonal, A::Union{LowerTriangular,UpperTriangular})
 end
 
 function rdiv!(A::AbstractMatrix{T}, D::Diagonal{T}) where {T}
-    @assert !has_offset_axes(A)
+    require_one_based_indexing(A)
     dd = D.diag
     m, n = size(A)
     if (k = length(dd)) ≠ n
@@ -396,7 +396,7 @@ function kron(A::Diagonal{T1}, B::Diagonal{T2}) where {T1<:Number, T2<:Number}
 end
 
 function kron(A::Diagonal{T}, B::AbstractMatrix{S}) where {T<:Number, S<:Number}
-    @assert ! Base.has_offset_axes(B)
+    Base.require_one_based_indexing(B)
     (mA, nA) = size(A); (mB, nB) = size(B)
     R = zeros(Base.promote_op(*, T, S), mA * mB, nA * nB)
     m = 1
@@ -415,7 +415,7 @@ function kron(A::Diagonal{T}, B::AbstractMatrix{S}) where {T<:Number, S<:Number}
 end
 
 function kron(A::AbstractMatrix{T}, B::Diagonal{S}) where {T<:Number, S<:Number}
-    @assert ! has_offset_axes(A)
+    require_one_based_indexing(A)
     (mA, nA) = size(A); (mB, nB) = size(B)
     R = zeros(promote_op(*, T, S), mA * mB, nA * nB)
     m = 1
@@ -451,8 +451,8 @@ function diag(D::Diagonal, k::Integer=0)
             "and at most $(size(D, 2)) for an $(size(D, 1))-by-$(size(D, 2)) matrix")))
     end
 end
-tr(D::Diagonal) = sum(D.diag)
-det(D::Diagonal) = prod(D.diag)
+tr(D::Diagonal) = sum(tr, D.diag)
+det(D::Diagonal) = prod(det, D.diag)
 logdet(D::Diagonal{<:Real}) = sum(log, D.diag)
 function logdet(D::Diagonal{<:Complex}) # make sure branch cut is correct
     z = sum(log, D.diag)
@@ -481,7 +481,7 @@ function ldiv!(D::Diagonal, B::StridedVecOrMat)
             if di == 0
                 throw(SingularException(i))
             end
-            B[i,j] /= di
+            B[i,j] = di \ B[i,j]
         end
     end
     return B
@@ -528,11 +528,11 @@ eigvals(D::Diagonal{<:Number}; permute::Bool=true, scale::Bool=true) = D.diag
 eigvals(D::Diagonal; permute::Bool=true, scale::Bool=true) =
     [eigvals(x) for x in D.diag] #For block matrices, etc.
 eigvecs(D::Diagonal) = Matrix{eltype(D)}(I, size(D))
-function eigen(D::Diagonal; permute::Bool=true, scale::Bool=true)
+function eigen(D::Diagonal; permute::Bool=true, scale::Bool=true, sortby::Union{Function,Nothing}=nothing)
     if any(!isfinite, D.diag)
         throw(ArgumentError("matrix contains Infs or NaNs"))
     end
-    Eigen(eigvals(D), eigvecs(D))
+    Eigen(sorteig!(eigvals(D), eigvecs(D), sortby)...)
 end
 
 #Singular system
