@@ -18,30 +18,75 @@ preview_info() = printstyled("───── Preview mode ─────\n"; c
 
 include("generate.jl")
 
-function check_package_name(x::AbstractString)
+function check_package_name(x::AbstractString, mode=nothing)
     if !(occursin(Pkg.REPLMode.name_re, x))
-         pkgerror("$x is not a valid packagename")
+        message = "$x is not a valid packagename."
+        if mode !== nothing && any(occursin.(['\\','/'], x)) # maybe a url or a path
+            message *= "\nThe argument appears to be a URL or path, perhaps you meant " *
+                "`Pkg.$mode(PackageSpec(url=\"...\"))` or `Pkg.$mode(PackageSpec(path=\"...\"))`."
+        end
+        pkgerror(message)
     end
     return PackageSpec(x)
 end
 
-add_or_develop(pkg::Union{AbstractString, PackageSpec}; kwargs...) = add_or_develop([pkg]; kwargs...)
-add_or_develop(pkgs::Vector{<:AbstractString}; kwargs...) =
-    add_or_develop([check_package_name(pkg) for pkg in pkgs]; kwargs...)
-add_or_develop(pkgs::Vector{PackageSpec}; kwargs...)      = add_or_develop(Context(), pkgs; kwargs...)
-
-function add_or_develop(ctx::Context, pkgs::Vector{PackageSpec}; mode::Symbol, shared::Bool=true, kwargs...)
-    pkgs = deepcopy(pkgs)  # deepcopy for avoid mutating PackageSpec members
+develop(pkg::Union{AbstractString, PackageSpec}; kwargs...) = develop([pkg]; kwargs...)
+develop(pkgs::Vector{<:AbstractString}; kwargs...) =
+    develop([check_package_name(pkg, :develop) for pkg in pkgs]; kwargs...)
+develop(pkgs::Vector{PackageSpec}; kwargs...)      = develop(Context(), pkgs; kwargs...)
+function develop(ctx::Context, pkgs::Vector{PackageSpec};
+                 shared::Bool=true, strict::Bool=false, kwargs...)
+    pkgs = deepcopy(pkgs) # deepcopy for avoid mutating PackageSpec members
     Context!(ctx; kwargs...)
-    # if julia is passed as a package the solver gets tricked;
-    # this catches the error early on
-    any(pkg -> (pkg.name == "julia"), pkgs) &&
-        pkgerror("Trying to $mode julia as a package")
+
+    for pkg in pkgs
+        # if julia is passed as a package the solver gets tricked
+        pkg.name != "julia" || pkgerror("Trying to develop julia as a package")
+        pkg.repo.rev === nothing || pkgerror("git revision can not be given to `develop`")
+        pkg.name !== nothing || pkg.uuid !== nothing || pkg.repo.url !== nothing ||
+            pkgerror("A package must be specified by `name`, `uuid`, `url`, or `path`.")
+        pkg.version == VersionSpec() ||
+            pkgerror("Can not specify version when tracking a repo.")
+    end
 
     ctx.preview && preview_info()
-    new_git = mode == :develop ?
-        handle_repos_develop!(ctx, pkgs, shared) :
-        handle_repos_add!(ctx, pkgs; upgrade_or_add=true)
+
+    new_git = handle_repos_develop!(ctx, pkgs, shared)
+
+    any(pkg -> Types.collides_with_project(ctx.env, pkg), pkgs) &&
+        pkgerror("Cannot `develop` package with the same name or uuid as the project")
+
+    Operations.develop(ctx, pkgs, new_git; strict=strict)
+    ctx.preview && preview_info()
+    return
+end
+
+add(pkg::Union{AbstractString, PackageSpec}; kwargs...) = add([pkg]; kwargs...)
+add(pkgs::Vector{<:AbstractString}; kwargs...) =
+    add([check_package_name(pkg, :add) for pkg in pkgs]; kwargs...)
+add(pkgs::Vector{PackageSpec}; kwargs...)      = add(Context(), pkgs; kwargs...)
+function add(ctx::Context, pkgs::Vector{PackageSpec}; strict::Bool=false, kwargs...)
+    pkgs = deepcopy(pkgs)  # deepcopy for avoid mutating PackageSpec members
+    Context!(ctx; kwargs...)
+
+    for pkg in pkgs
+        # if julia is passed as a package the solver gets tricked; this catches the error early on
+        pkg.name == "julia" && pkgerror("Trying to add julia as a package")
+        pkg.name !== nothing || pkg.uuid !== nothing || pkg.repo.url !== nothing ||
+            pkgerror("A package must be specified by `name`, `uuid`, `url`, or `path`.")
+        if (pkg.repo.url !== nothing || pkg.repo.rev !== nothing)
+            pkg.version == VersionSpec() ||
+                pkgerror("Can not specify version when tracking a repo.")
+        end
+    end
+
+    ctx.preview && preview_info()
+    Types.update_registries(ctx)
+
+    repo_pkgs = [pkg for pkg in pkgs if (pkg.repo.url !== nothing || pkg.repo.rev !== nothing)]
+    new_git = handle_repos_add!(ctx, repo_pkgs)
+    # repo + unpinned -> name, uuid, repo.rev, repo.url, tree_hash
+    # repo + pinned -> name, uuid, tree_hash
 
     project_deps_resolve!(ctx.env, pkgs)
     registry_resolve!(ctx.env, pkgs)
@@ -51,63 +96,67 @@ function add_or_develop(ctx::Context, pkgs::Vector{PackageSpec}; mode::Symbol, s
     any(pkg -> Types.collides_with_project(ctx.env, pkg), pkgs) &&
         pkgerror("Cannot $mode package with the same name or uuid as the project")
 
-    Operations.add_or_develop(ctx, pkgs; new_git=new_git, mode=mode)
+    Operations.add(ctx, pkgs, new_git; strict=strict)
     ctx.preview && preview_info()
     return
 end
 
-add(args...; kwargs...) = add_or_develop(args...; mode = :add, kwargs...)
-develop(args...; shared=true, kwargs...) = add_or_develop(args...; mode = :develop, shared = shared, kwargs...)
-
 rm(pkg::Union{AbstractString, PackageSpec}; kwargs...) = rm([pkg]; kwargs...)
 rm(pkgs::Vector{<:AbstractString}; kwargs...)          = rm([PackageSpec(pkg) for pkg in pkgs]; kwargs...)
-rm(pkgs::Vector{PackageSpec}; kwargs...)       = rm(Context(), pkgs; kwargs...)
+rm(pkgs::Vector{PackageSpec}; kwargs...)               = rm(Context(), pkgs; kwargs...)
 
 function rm(ctx::Context, pkgs::Vector{PackageSpec}; mode=PKGMODE_PROJECT, kwargs...)
     pkgs = deepcopy(pkgs)  # deepcopy for avoid mutating PackageSpec members
+    foreach(pkg -> pkg.mode = mode, pkgs)
+
     for pkg in pkgs
-        # TODO only overwrite pkg.mode if default value ?
-        pkg.mode = mode
+        pkg.name !== nothing || pkg.uuid !== nothing ||
+            pkgerror("Must specify package by either `name` or `uuid`.")
+        if !(pkg.version == VersionSpec() && pkg.pinned == false &&
+             pkg.tree_hash === nothing && pkg.repo.url === nothing &&
+             pkg.repo.rev === nothing && pkg.path === nothing)
+            pkgerror("Package may only be specified by either `name` or `uuid`")
+        end
     end
 
     Context!(ctx; kwargs...)
     ctx.preview && preview_info()
+
     project_deps_resolve!(ctx.env, pkgs)
     manifest_resolve!(ctx.env, pkgs)
+    ensure_resolved(ctx.env, pkgs)
+
     Operations.rm(ctx, pkgs)
     ctx.preview && preview_info()
     return
 end
 
-up(ctx::Context; kwargs...)                    = up(ctx, PackageSpec[]; kwargs...)
-up(; kwargs...)                                = up(PackageSpec[]; kwargs...)
+up(ctx::Context; kwargs...)                            = up(ctx, PackageSpec[]; kwargs...)
+up(; kwargs...)                                        = up(PackageSpec[]; kwargs...)
 up(pkg::Union{AbstractString, PackageSpec}; kwargs...) = up([pkg]; kwargs...)
 up(pkgs::Vector{<:AbstractString}; kwargs...)          = up([PackageSpec(pkg) for pkg in pkgs]; kwargs...)
-up(pkgs::Vector{PackageSpec}; kwargs...)       = up(Context(), pkgs; kwargs...)
+up(pkgs::Vector{PackageSpec}; kwargs...)               = up(Context(), pkgs; kwargs...)
 
 function up(ctx::Context, pkgs::Vector{PackageSpec};
-            level::UpgradeLevel=UPLEVEL_MAJOR, mode::PackageMode=PKGMODE_PROJECT, do_update_registry=true, kwargs...)
+            level::UpgradeLevel=UPLEVEL_MAJOR, mode::PackageMode=PKGMODE_PROJECT,
+            update_registry::Bool=true, kwargs...)
     pkgs = deepcopy(pkgs)  # deepcopy for avoid mutating PackageSpec members
-    for pkg in pkgs
-        # TODO only override if they are not already set
-        pkg.mode = mode
-        pkg.version = level
-    end
+    foreach(pkg -> pkg.mode = mode, pkgs)
 
     Context!(ctx; kwargs...)
     ctx.preview && preview_info()
-    if do_update_registry
+    if update_registry
         Types.clone_default_registries()
-        Types.update_registries(ctx)
+        Types.update_registries(ctx; force=true)
     end
     if isempty(pkgs)
         if mode == PKGMODE_PROJECT
             for (name::String, uuid::UUID) in ctx.env.project.deps
-                push!(pkgs, PackageSpec(name=name, uuid=uuid, version=level))
+                push!(pkgs, PackageSpec(name=name, uuid=uuid))
             end
         elseif mode == PKGMODE_MANIFEST
             for (uuid, entry) in ctx.env.manifest
-                push!(pkgs, PackageSpec(name=entry.name, uuid=uuid, version=level))
+                push!(pkgs, PackageSpec(name=entry.name, uuid=uuid))
             end
         end
     else
@@ -115,22 +164,31 @@ function up(ctx::Context, pkgs::Vector{PackageSpec};
         manifest_resolve!(ctx.env, pkgs)
         ensure_resolved(ctx.env, pkgs)
     end
-    Operations.up(ctx, pkgs)
+    Operations.up(ctx, pkgs, level)
     ctx.preview && preview_info()
     return
 end
 
 resolve(ctx::Context=Context()) =
-    up(ctx, level=UPLEVEL_FIXED, mode=PKGMODE_MANIFEST, do_update_registry=false)
+    up(ctx, level=UPLEVEL_FIXED, mode=PKGMODE_MANIFEST, update_registry=false)
 
 pin(pkg::Union{AbstractString, PackageSpec}; kwargs...) = pin([pkg]; kwargs...)
 pin(pkgs::Vector{<:AbstractString}; kwargs...)          = pin([PackageSpec(pkg) for pkg in pkgs]; kwargs...)
-pin(pkgs::Vector{PackageSpec}; kwargs...)       = pin(Context(), pkgs; kwargs...)
+pin(pkgs::Vector{PackageSpec}; kwargs...)               = pin(Context(), pkgs; kwargs...)
 
 function pin(ctx::Context, pkgs::Vector{PackageSpec}; kwargs...)
     pkgs = deepcopy(pkgs)  # deepcopy for avoid mutating PackageSpec members
     Context!(ctx; kwargs...)
     ctx.preview && preview_info()
+
+    for pkg in pkgs
+        pkg.name !== nothing || pkg.uuid !== nothing ||
+            pkgerror("Must specify package by either `name` or `uuid`.")
+        pkg.repo.url === nothing || pkgerror("Can not specify `repo` url")
+        pkg.repo.rev === nothing || pkgerror("Can not specify `repo` rev")
+    end
+
+    foreach(pkg -> pkg.mode = PKGMODE_PROJECT, pkgs)
     project_deps_resolve!(ctx.env, pkgs)
     ensure_resolved(ctx.env, pkgs)
     Operations.pin(ctx, pkgs)
@@ -146,55 +204,48 @@ function free(ctx::Context, pkgs::Vector{PackageSpec}; kwargs...)
     pkgs = deepcopy(pkgs)  # deepcopy for avoid mutating PackageSpec members
     Context!(ctx; kwargs...)
     ctx.preview && preview_info()
-    registry_resolve!(ctx.env, pkgs)
-    uuids_in_registry = UUID[]
+
     for pkg in pkgs
-        pkg.mode = PKGMODE_MANIFEST
-    end
-    for pkg in pkgs
-        has_uuid(pkg) && push!(uuids_in_registry, pkg.uuid)
-    end
-    manifest_resolve!(ctx.env, pkgs)
-    ensure_resolved(ctx.env, pkgs)
-    # Every non pinned package that is freed need to be in a registry
-    for pkg in pkgs
-        entry = manifest_info(ctx.env, pkg.uuid)
-        if !entry.pinned && !(pkg.uuid in uuids_in_registry)
-            pkgerror("cannot free an unpinned package that does not exist in a registry")
+        pkg.name !== nothing || pkg.uuid !== nothing ||
+            pkgerror("Must specify package by either `name` or `uuid`.")
+        if !(pkg.version == VersionSpec() && pkg.pinned == false &&
+             pkg.tree_hash === nothing && pkg.repo.url === nothing &&
+             pkg.repo.rev === nothing && pkg.path === nothing)
+            pkgerror("Package may only be specified by either `name` or `uuid`")
         end
     end
+
+    foreach(pkg -> pkg.mode = PKGMODE_MANIFEST, pkgs)
+    manifest_resolve!(ctx.env, pkgs)
+    ensure_resolved(ctx.env, pkgs)
+
+    find_registered!(ctx.env, UUID[pkg.uuid for pkg in pkgs])
     Operations.free(ctx, pkgs)
     return
 end
 
-
-
-test(;kwargs...)                                  = test(PackageSpec[]; kwargs...)
-test(pkg::Union{AbstractString, PackageSpec}; kwargs...)  = test([pkg]; kwargs...)
-test(pkgs::Vector{<:AbstractString}; kwargs...)           = test([PackageSpec(pkg) for pkg in pkgs]; kwargs...)
-test(pkgs::Vector{PackageSpec}; kwargs...)        = test(Context(), pkgs; kwargs...)
-
-function test(ctx::Context, pkgs::Vector{PackageSpec}; coverage=false, kwargs...)
-    pkgs = deepcopy(pkgs)  # deepcopy for avoid mutating PackageSpec members
+test(;kwargs...)                                         = test(PackageSpec[]; kwargs...)
+test(pkg::Union{AbstractString, PackageSpec}; kwargs...) = test([pkg]; kwargs...)
+test(pkgs::Vector{<:AbstractString}; kwargs...)          = test([PackageSpec(pkg) for pkg in pkgs]; kwargs...)
+test(pkgs::Vector{PackageSpec}; kwargs...)               = test(Context(), pkgs; kwargs...)
+function test(ctx::Context, pkgs::Vector{PackageSpec};
+              coverage=false, test_fn=nothing, kwargs...)
+    pkgs = deepcopy(pkgs) # deepcopy for avoid mutating PackageSpec members
     Context!(ctx; kwargs...)
     ctx.preview && preview_info()
     if isempty(pkgs)
-        # TODO: Allow this?
-        ctx.env.pkg == nothing && pkgerror("trying to test unnamed project")
+        ctx.env.pkg === nothing && pkgerror("trying to test unnamed project") #TODO Allow this?
         push!(pkgs, ctx.env.pkg)
+    else
+        project_resolve!(ctx.env, pkgs)
+        project_deps_resolve!(ctx.env, pkgs)
+        manifest_resolve!(ctx.env, pkgs)
+        ensure_resolved(ctx.env, pkgs)
     end
-    project_resolve!(ctx.env, pkgs)
-    project_deps_resolve!(ctx.env, pkgs)
-    manifest_resolve!(ctx.env, pkgs)
-    ensure_resolved(ctx.env, pkgs)
-    if !ctx.preview && (Operations.any_package_not_installed(ctx) || !isfile(ctx.env.manifest_file))
-        Pkg.instantiate(ctx)
-    end
-    Operations.test(ctx, pkgs; coverage=coverage)
+    Operations.test(ctx, pkgs; coverage=coverage, test_fn=test_fn)
     return
 end
 
-
 installed() = __installed(PKGMODE_PROJECT)
 function __installed(mode::PackageMode=PKGMODE_MANIFEST)
     diffs = Display.status(Context(), PackageSpec[], mode=mode, use_as_api=true)
@@ -239,9 +290,9 @@ function gc(ctx::Context=Context(); kwargs...)
         manifest == nothing && continue
         new_usage[manifestfile] = [Dict("time" => date)]
         for (uuid, entry) in manifest
-            if entry.repo.tree_sha !== nothing
+            if entry.tree_hash !== nothing
                 push!(paths_to_keep,
-                      Operations.find_installed(entry.name, uuid, entry.repo.tree_sha))
+                      Operations.find_installed(entry.name, uuid, entry.tree_hash))
             end
         end
     end
@@ -255,7 +306,7 @@ function gc(ctx::Context=Context(); kwargs...)
                 if isdir(joinpath(packagedir, name))
                     for slug in readdir(joinpath(packagedir, name))
                         versiondir = joinpath(packagedir, name, slug)
-                        if !(versiondir in paths_to_keep)
+                        if !(versiondir in paths_to_keep) && isdir(versiondir)
                             push!(paths_to_delete, versiondir)
                         end
                     end
@@ -323,27 +374,6 @@ function gc(ctx::Context=Context(); kwargs...)
     return
 end
 
-
-function _get_deps!(ctx::Context, pkgs::Vector{PackageSpec}, uuids::Vector{UUID})
-    for pkg in pkgs
-        pkg.uuid in keys(ctx.stdlibs) && continue
-        pkg.uuid in uuids && continue
-        push!(uuids, pkg.uuid)
-        if Types.is_project(ctx.env, pkg)
-            pkgs = [PackageSpec(name, uuid) for (name, uuid) in ctx.env.project.deps]
-        else
-            info = manifest_info(ctx.env, pkg.uuid)
-            if info === nothing
-                pkgerror("could not find manifest info for package with uuid: $(pkg.uuid)")
-            end
-            pkgs = [PackageSpec(name, uuid) for (name, uuid) in info.deps]
-        end
-        _get_deps!(ctx, pkgs, uuids)
-    end
-    return
-end
-
-
 build(pkgs...; kwargs...) = build([PackageSpec(pkg) for pkg in pkgs]; kwargs...)
 build(pkg::Array{Union{}, 1}; kwargs...) = build(PackageSpec[]; kwargs...)
 build(pkg::PackageSpec; kwargs...) = build([pkg]; kwargs...)
@@ -362,21 +392,11 @@ function build(ctx::Context, pkgs::Vector{PackageSpec}; verbose=false, kwargs...
             end
         end
     end
-    for pkg in pkgs
-        pkg.mode = PKGMODE_MANIFEST
-    end
     project_resolve!(ctx.env, pkgs)
+    foreach(pkg -> pkg.mode = PKGMODE_MANIFEST, pkgs)
     manifest_resolve!(ctx.env, pkgs)
     ensure_resolved(ctx.env, pkgs)
-    if !ctx.preview && (Operations.any_package_not_installed(ctx) || !isfile(ctx.env.manifest_file))
-        Pkg.instantiate(ctx)
-    end
-    uuids = UUID[]
-    _get_deps!(ctx, pkgs, uuids)
-    length(uuids) == 0 && (@info("no packages to build"); return)
-    Operations.build_versions(ctx, uuids; might_need_to_resolve=true, verbose=verbose)
-    ctx.preview && preview_info()
-    return
+    Operations.build(ctx, pkgs, verbose)
 end
 
 #####################################
@@ -388,7 +408,7 @@ function clone(url::String, name::String = "")
     if !isempty(name)
         ctx.old_pkg2_clone_name = name
     end
-    develop(ctx, [Pkg.REPLMode.parse_package(url; add_or_develop=true)])
+    develop(ctx, [Pkg.REPLMode.parse_package_identifier(url; add_or_develop=true)])
 end
 
 function dir(pkg::String, paths::AbstractString...)
@@ -426,45 +446,51 @@ function precompile(ctx::Context)
         end
         if stale
             printpkgstyle(ctx, :Precompiling, pkg.name)
-            Base.compilecache(pkg, sourcepath)
+            try
+                Base.compilecache(pkg, sourcepath)
+            catch
+            end
         end
     end
     nothing
 end
 
 instantiate(; kwargs...) = instantiate(Context(); kwargs...)
-function instantiate(ctx::Context; manifest::Union{Bool, Nothing}=nothing, kwargs...)
+function instantiate(ctx::Context; manifest::Union{Bool, Nothing}=nothing,
+                     update_registry::Bool=true, kwargs...)
     Context!(ctx; kwargs...)
     if (!isfile(ctx.env.manifest_file) && manifest == nothing) || manifest == false
-        up(ctx)
+        up(ctx; update_registry=update_registry)
         return
     end
     if !isfile(ctx.env.manifest_file) && manifest == true
         pkgerror("manifest at $(ctx.env.manifest_file) does not exist")
     end
     Operations.prune_manifest(ctx.env)
+    for (name, uuid) in ctx.env.project.deps
+        get(ctx.env.manifest, uuid, nothing) === nothing || continue
+        pkgerror("`$name` is a direct dependency, but does not appear in the manifest.",
+                 " If you intend `$name` to be a direct dependency, run `Pkg.resolve()` to populate the manifest.",
+                 " Otherwise, remove `$name` with `Pkg.rm(\"$name\")`.",
+                 " Finally, run `Pkg.instantiate()` again.")
+    end
     Types.update_registries(ctx)
-    urls = Dict{}
-    hashes = Dict{UUID,SHA1}()
-    urls = Dict{UUID,Vector{String}}()
     pkgs = PackageSpec[]
-    for (uuid, entry) in ctx.env.manifest
-        pkg = PackageSpec(name=entry.name, uuid=uuid, path=entry.path,
-                          version = entry.version !== nothing ? VersionNumber(entry.version) : VersionSpec())
-        push!(pkgs, pkg)
-        pkg.uuid in keys(ctx.stdlibs) && continue
-        pkg.path !== nothing && continue
-        urls[pkg.uuid] = String[]
-        hashes[pkg.uuid] = entry.repo.tree_sha
-        entry.repo.url !== nothing && (pkg.repo = entry.repo)
+    Operations.load_all_deps!(ctx, pkgs)
+    Operations.check_registered(ctx, pkgs)
+    new_git = UUID[]
+    for pkg in pkgs
+        pkg.repo.url !== nothing || continue
+        sourcepath = Operations.source_path(pkg)
+        isdir(sourcepath) && continue
+        # download repo at tree hash
+        push!(new_git, pkg.uuid)
+        clonepath = Types.clone_path!(pkg.repo.url)
+        tmp_source = Types.repo_checkout(clonepath, string(pkg.tree_hash))
+        mkpath(sourcepath)
+        mv(tmp_source, sourcepath; force=true)
     end
-    _, urls_ref = Operations.version_data!(ctx, pkgs)
-    for (uuid, url) in urls_ref
-        append!(urls[uuid], url)
-        urls[uuid] = unique(urls[uuid])
-    end
-    new_git = handle_repos_add!(ctx, pkgs; upgrade_or_add=false)
-    new_apply = Operations.apply_versions(ctx, pkgs, hashes, urls)
+    new_apply = Operations.download_source(ctx, pkgs)
     Operations.build_versions(ctx, union(new_apply, new_git))
 end
 
@@ -479,6 +505,9 @@ status(pkgs::Vector{PackageSpec}; mode=PKGMODE_PROJECT) = status(Context(), pkgs
 function status(ctx::Context, pkgs::Vector{PackageSpec}; mode=PKGMODE_PROJECT)
     project_resolve!(ctx.env, pkgs)
     project_deps_resolve!(ctx.env, pkgs)
+    if mode === PKGMODE_MANIFEST
+        foreach(pkg -> pkg.mode = PKGMODE_MANIFEST, pkgs)
+    end
     manifest_resolve!(ctx.env, pkgs)
     ensure_resolved(ctx.env, pkgs)
     Pkg.Display.status(ctx, pkgs, mode=mode)
@@ -486,7 +515,12 @@ function status(ctx::Context, pkgs::Vector{PackageSpec}; mode=PKGMODE_PROJECT)
 end
 
 
-activate() = (Base.ACTIVE_PROJECT[] = nothing)
+function activate()
+    Base.ACTIVE_PROJECT[] = nothing
+    p = Base.active_project()
+    p === nothing || printpkgstyle(Context(), :Activating, "environment at $(pathrepr(p))")
+    return nothing
+end
 function _activate_dep(dep_name::AbstractString)
     Base.active_project() === nothing && return
     env = nothing
@@ -516,7 +550,6 @@ function activate(path::AbstractString; shared::Bool=false)
             fullpath = _activate_dep(path)
             if fullpath === nothing
                 fullpath = abspath(path)
-                @info("activating new environment at $(Base.contractuser(fullpath)).")
             end
         end
     else
@@ -534,13 +567,26 @@ function activate(path::AbstractString; shared::Bool=false)
         # unless the shared environment already exists, place it in the first depots
         if !isdir(fullpath)
             fullpath = joinpath(Pkg.envdir(Pkg.depots1()), path)
-            @info("activating new environment at $(Base.contractuser(fullpath)).")
         end
     end
     Base.ACTIVE_PROJECT[] = Base.load_path_expand(fullpath)
+    p = Base.active_project()
+    if p !== nothing
+        n = ispath(p) ? "" : "new "
+        printpkgstyle(Context(), :Activating, "$(n)environment at $(pathrepr(p))")
+    end
+    return nothing
 end
 
-setprotocol!(proto::Union{Nothing, AbstractString}=nothing) = GitTools.setprotocol!(proto)
+function setprotocol!(;
+    domain::AbstractString="github.com",
+    protocol::Union{Nothing, AbstractString}=nothing
+)
+    GitTools.setprotocol!(domain=domain, protocol=protocol)
+    return nothing
+end
+
+@deprecate setprotocol!(proto::Union{Nothing, AbstractString}) setprotocol!(protocol = proto) false
 
 # API constructor
 function Package(;name::Union{Nothing,AbstractString} = nothing,
@@ -551,12 +597,11 @@ function Package(;name::Union{Nothing,AbstractString} = nothing,
         pkgerror("cannot specify both a path and url")
     url !== nothing && version !== nothing &&
         pkgerror("`version` can not be given with `url`, use `rev` instead")
-    repo = (url === nothing && path === nothing && rev === nothing) ?
-        nothing :
-        Types.GitRepo(rev = rev, url = url !== nothing ? url : path)
+    repo = Types.GitRepo(rev = rev, url = url !== nothing ? url : path)
     version = version === nothing ? VersionSpec() : VersionSpec(version)
     uuid isa String && (uuid = UUID(uuid))
-    PackageSpec(name, uuid, version, mode, nothing, PKGSPEC_NOTHING, repo)
+    PackageSpec(;name=name, uuid=uuid, version=version, mode=mode, path=nothing,
+                special_action=PKGSPEC_NOTHING, repo=repo, tree_hash=nothing)
 end
 Package(name::AbstractString) = PackageSpec(name)
 Package(name::AbstractString, uuid::UUID) = PackageSpec(name, uuid)
