@@ -80,14 +80,15 @@ const UpgradeLevel = Types.UpgradeLevel
     Pkg.add(pkg::Union{String, Vector{String}})
     Pkg.add(pkg::Union{PackageSpec, Vector{PackageSpec}})
 
-Add a package to the current project. This package will be available using the
-`import` and `using` keywords in the Julia REPL and if the current project is
+Add a package to the current project. This package will be available by using the
+`import` and `using` keywords in the Julia REPL, and if the current project is
 a package, also inside that package.
 
 # Examples
 ```julia
 Pkg.add("Example") # Add a package from registry
-Pkg.add(PackageSpec(name="Example", version="0.3")) # Specify version
+Pkg.add(PackageSpec(name="Example", version="0.3")) # Specify version; latest release in the 0.3 series
+Pkg.add(PackageSpec(name="Example", version="0.3.1")) # Specify version; exact release
 Pkg.add(PackageSpec(url="https://github.com/JuliaLang/Example.jl", rev="master")) # From url to remote gitrepo
 Pkg.add(PackageSpec(url="/remote/mycompany/juliapackages/OurPackage"))` # From path to local gitrepo
 ```
@@ -114,7 +115,7 @@ const rm = API.rm
     Pkg.update(pkg::Union{PackageSpec, Vector{PackageSpec}})
 
 Update a package `pkg`. If no posistional argument is given, update all packages in the manifest if `mode` is `PKGMODE_MANIFEST` and packages in both manifest and project if `mode` is `PKGMODE_PROJECT`.
-If no positional argument is given `level` can be used to control what how much packages are allowed to be upgraded (major, minor, patch, fixed).
+If no positional argument is given, `level` can be used to control by how much packages are allowed to be upgraded (major, minor, patch, fixed).
 
 See also [`PackageSpec`](@ref), [`PackageMode`](@ref), [`UpgradeLevel`](@ref).
 """
@@ -126,15 +127,15 @@ const update = API.up
     Pkg.test(pkg::Union{String, Vector{String}; coverage::Bool=false)
     Pkg.test(pkgs::Union{PackageSpec, Vector{PackageSpec}}; coverage::Bool=false)
 
-Run the tests for package `pkg` or if no positional argument is given to `test`,
-the current project is tested (which thus needs to be a package).
-A package is tested by running its `test/runtests.jl` file.
+Run the tests for package `pkg`, or for the current project (which thus needs to be a package) if no
+positional argument is given to `Pkg.test`. A package is tested by running its
+`test/runtests.jl` file.
 
 The tests are run by generating a temporary environment with only `pkg` and its (recursive) dependencies
-(recursively) in it. If a manifest exist, the versions in that manifest is used, otherwise
-a feasible set of package are resolved and installed.
+in it. If a manifest exists, the versions in that manifest are used, otherwise
+a feasible set of packages is resolved and installed.
 
-During the test, test-specific dependencies are active, which are
+During the tests, test-specific dependencies are active, which are
 given in the project file as e.g.
 
 ```
@@ -146,7 +147,7 @@ test = ["Test"]
 ```
 
 Coverage statistics for the packages may be generated by
-passing `coverage=true`. The default behavior is not to run coverage.
+passing `coverage=true`. The default behavior is to not run coverage.
 
 The tests are executed in a new process with `check-bounds=yes` and by default `startup-file=no`.
 If using the startup file (`~/.julia/config/startup.jl`) is desired, start julia with `--startup-file=yes`.
@@ -170,11 +171,11 @@ const gc = API.gc
     Pkg.build(pkg::Union{String, Vector{String}}; verbose = false)
     Pkg.build(pkgs::Union{PackageSpec, Vector{PackageSpec}}; verbose = false)
 
-Run the build script in `deps/build.jl` for `pkg` and all of the dependencies in
+Run the build script in `deps/build.jl` for `pkg` and all of its dependencies in
 depth-first recursive order.
 If no argument is given to `build`, the current project is built, which thus needs
 to be a package.
-This function is called automatically one any package that gets installed
+This function is called automatically on any package that gets installed
 for the first time.
 `verbose = true` prints the build output to `stdout`/`stderr` instead of
 redirecting to the `build.log` file.
@@ -186,18 +187,25 @@ const installed = API.installed
 
 """
     Pkg.pin(pkg::Union{String, Vector{String}})
-    Pkg.pin(pkgs::Union{Packagespec, Vector{Packagespec}})
+    Pkg.pin(pkgs::Union{PackageSpec, Vector{PackageSpec}})
 
-Pin a package to the current version (or the one given in the `packagespec` or a certain
+Pin a package to the current version (or the one given in the `PackageSpec`) or to a certain
 git revision. A pinned package is never updated.
+
+# Examples
+```julia
+Pkg.pin("Example")
+Pkg.pin(PackageSpec(name="Example", version="0.3.1"))
+```
 """
 const pin = API.pin
 
 """
     Pkg.free(pkg::Union{String, Vector{String}})
-    Pkg.free(pkgs::Union{Packagespec, Vector{Packagespec}})
+    Pkg.free(pkgs::Union{PackageSpec, Vector{PackageSpec}})
 
-Free a package which removes a `pin` if it exists, or if the package is tracking a path,
+If `pkg` is pinned, remove the pin.
+If `pkg` is tracking a path,
 e.g. after [`Pkg.develop`](@ref), go back to tracking registered versions.
 
 # Examples
@@ -214,8 +222,8 @@ const free = API.free
     Pkg.develop(pkgs::Union{Packagespec, Vector{Packagespec}})
 
 Make a package available for development by tracking it by path.
-If `pkg` is given with only a name or by a URL the packages will be downloaded
-to the location by the environment variable `JULIA_PKG_DEVDIR` with
+If `pkg` is given with only a name or by a URL, the package will be downloaded
+to the location specified by the environment variable `JULIA_PKG_DEVDIR`, with
 `.julia/dev` as the default.
 
 If `pkg` is given as a local path, the package at that path will be tracked.
@@ -243,9 +251,9 @@ const generate = API.generate
 """
     Pkg.instantiate()
 
-If a `Manifest.toml` file exist in the current project, download all
+If a `Manifest.toml` file exists in the current project, download all
 the packages declared in that manifest.
-Else, resolve a set of feasible packages from the `Project.toml` files
+Otherwise, resolve a set of feasible packages from the `Project.toml` files
 and install them.
 """
 const instantiate = API.instantiate
@@ -253,7 +261,7 @@ const instantiate = API.instantiate
 """
     Pkg.resolve()
 
-Update the current manifest with eventual changes to the dependency graph
+Update the current manifest with potential changes to the dependency graph
 from packages that are tracking a path.
 """
 const resolve = API.resolve
@@ -262,10 +270,10 @@ const resolve = API.resolve
     Pkg.status([pkgs...]; mode::PackageMode=PKGMODE_PROJECT)
 
 Print out the status of the project/manifest.
-If `mode` is `PKGMODE_PROJECT` prints out status about only those packages
-that are in the project (explicitly added). If `mode` is `PKGMODE_MANIFEST`
-also print for those in the manifest (recursive dependencies). If there are
-any packages listed as arguments the output will be limited to those packages.
+If `mode` is `PKGMODE_PROJECT`, print out status only about the packages
+that are in the project (explicitly added). If `mode` is `PKGMODE_MANIFEST`,
+print status also about those in the manifest (recursive dependencies). If there are
+any packages listed as arguments, the output will be limited to those packages.
 
 !!! compat "Julia 1.1"
     `Pkg.status` with package arguments requires at least Julia 1.1.
@@ -286,7 +294,7 @@ The logic for what path is activated is as follows:
   * If `s` is an existing path, then activate the environment at that path.
   * If `s` is a package in the current project and `s` is tracking a path, then
     activate the environment at the tracked path.
-  * Else, `s` is interpreted as a non-existing path, activate that path.
+  * Otherwise, `s` is interpreted as a non-existing path, which is then activated.
 
 If no argument is given to `activate`, then activate the home project.
 The home project is specified by either the `--project` command line option to
@@ -310,18 +318,18 @@ A `PackageSpec` is a representation of a package with various metadata.
 This includes:
 
   * The `name` of the package.
-  * The package unique `uuid`.
-  * A `version` (for example when adding a package. When upgrading, can also be an instance of
-   the enum [`UpgradeLevel`](@ref)
-  * A `url` and an optional git `rev`ision. `rev` could be a branch name or a git commit SHA.
-  * A local path `path`. This is equivalent to using the `url` argument but can be more descriptive.
-  * A `mode`, which is an instance of the enum [`PackageMode`](@ref) which can be either `PKGMODE_PROJECT` or
-   `PKGMODE_MANIFEST`, defaults to `PKGMODE_PROJECT`. Used in e.g. [`Pkg.rm`](@ref).
+  * The package's unique `uuid`.
+  * A `version` (for example when adding a package). When upgrading, can also be an instance of
+   the enum [`UpgradeLevel`](@ref).
+  * A `url` and an optional git `rev`ision. `rev` can be a branch name or a git commit SHA1.
+  * A local `path`. This is equivalent to using the `url` argument but can be more descriptive.
+  * A `mode`, which is an instance of the enum [`PackageMode`](@ref), with possible values `PKGMODE_PROJECT`
+   (the default) or `PKGMODE_MANIFEST`. Used in e.g. [`Pkg.rm`](@ref).
 
 Most functions in Pkg take a `Vector` of `PackageSpec` and do the operation on all the packages
 in the vector.
 
-Below is a comparison between the REPL version and the `PackageSpec` version:
+Below is a comparison between the REPL version and the API version:
 
 | `REPL`               | `API`                                                 |
 |:---------------------|:------------------------------------------------------|
@@ -337,11 +345,21 @@ Below is a comparison between the REPL version and the `PackageSpec` version:
 const PackageSpec = API.Package
 
 """
-    Pkg.setprotocol!(proto::Union{Nothing, AbstractString}=nothing)
+    setprotocol!(;
+        domain::AbstractString = "github.com",
+        protocol::Union{Nothing, AbstractString}=nothing
+    )
 
-Set the protocol used to access GitHub-hosted packages when `add`ing a url or `develop`ing a package.
-Defaults to delegating the choice to the package developer (`proto == nothing`).
-Other choices for `proto` are `"https"` or `"git"`.
+Set the protocol used to access hosted packages when `add`ing a url or `develop`ing a package.
+Defaults to delegating the choice to the package developer (`protocol == nothing`).
+Other choices for `protocol` are `"https"` or `"git"`.
+
+# Examples
+```julia-repl
+julia> Pkg.setprotocol!(domain = "github.com", protocol = "ssh")
+
+julia> Pkg.setprotocol!(domain = "gitlab.mycompany.com")
+```
 """
 const setprotocol! = API.setprotocol!
 
@@ -360,7 +378,7 @@ on all the registries in the vector.
 
 # Examples
 
-Below is a comparison between the REPL version and the `RegistrySpec` version:
+Below is a comparison between the REPL version and the API version:
 
 | `REPL`               | `API`                                           |
 |:---------------------|:------------------------------------------------|
