@@ -132,16 +132,13 @@ temp_pkg_dir() do project_path
         @test isinstalled(TEST_PKG)
         Pkg.rm(TEST_PKG.name)
         @test !isinstalled(TEST_PKG)
-        # https://github.com/JuliaLang/Pkg.jl/issues/601
         pkgdir = joinpath(Pkg.depots1(), "packages")
-        touch(joinpath(pkgdir, ".DS_Store"))
         Pkg.gc()
-        rm(joinpath(pkgdir, ".DS_Store"))
         @test isempty(readdir(pkgdir))
     end
 
     @testset "package with wrong UUID" begin
-        @test_throws ResolverError Pkg.add(PackageSpec(TEST_PKG.name, UUID(UInt128(1))))
+        @test_throws PkgError Pkg.add(PackageSpec(TEST_PKG.name, UUID(UInt128(1))))
     end
 
     @testset "adding and upgrading different versions" begin
@@ -254,13 +251,13 @@ temp_pkg_dir() do project_path
         mktempdir() do devdir
             withenv("JULIA_PKG_DEVDIR" => devdir) do
                 try
-                    Pkg.setprotocol!("notarealprotocol")
+                    Pkg.setprotocol!(domain = "github.com", protocol = "notarealprotocol")
                     @test_throws PkgError Pkg.develop("Example")
-                    Pkg.setprotocol!("https")
+                    Pkg.setprotocol!(domain = "github.com", protocol = "https")
                     Pkg.develop("Example")
                     @test isinstalled(TEST_PKG)
                 finally
-                    Pkg.setprotocol!()
+                    Pkg.setprotocol!(domain = "github.com")
                 end
             end
         end
@@ -270,14 +267,26 @@ temp_pkg_dir() do project_path
                     https_url = "https://github.com/JuliaLang/Example.jl.git"
                     ssh_url = "ssh://git@github.com/JuliaLang/Example.jl.git"
                     @test Pkg.GitTools.normalize_url(https_url) == https_url
-                    Pkg.setprotocol!("ssh")
+                    Pkg.setprotocol!(domain = "github.com", protocol = "ssh")
                     @test Pkg.GitTools.normalize_url(https_url) == ssh_url
                     # TODO: figure out how to test this without
                     #       having to deploy a ssh key on github
                     #Pkg.develop("Example")
                     #@test isinstalled(TEST_PKG)
+
+                    https_url = "https://gitlab.example.com/example/Example.jl.git"
+                    ssh_url = "ssh://git@gitlab.example.com/example/Example.jl.git"
+
+                    @test Pkg.GitTools.normalize_url(https_url) == https_url
+                    Pkg.setprotocol!(domain = "gitlab.example.com", protocol = "ssh")
+                    @test Pkg.GitTools.normalize_url(https_url) == ssh_url
+
+                    @test_deprecated Pkg.setprotocol!("ssh")
+                    @test_deprecated Pkg.GitTools.setprotocol!("ssh")
+
                 finally
-                    Pkg.setprotocol!()
+                    Pkg.setprotocol!(domain = "github.com")
+                    Pkg.setprotocol!(domain = "gitlab.example.com")
                 end
             end
         end
@@ -352,10 +361,6 @@ temp_pkg_dir() do project_path
     end
 end
 
-@testset "parse package url win" begin
-    @test typeof(Pkg.REPLMode.parse_package("https://github.com/abc/ABC.jl"; add_or_develop=true)) == PackageSpec
-end
-
 @testset "preview generate" begin
     mktempdir() do tmp
         cd(tmp) do
@@ -553,6 +558,9 @@ end
         for (uuid, x) in a
             y = b[uuid]
             for property in propertynames(x)
+                # `other` caches the *whole* input dictionary. its ok to mutate the fields of
+                # the input dictionary if that field will eventually be overwriten on `write_manifest`
+                property == :other && continue
                 @test getproperty(x, property) == getproperty(y, property)
             end
         end
@@ -602,16 +610,188 @@ end
     end
 end
 
+@testset "issue #1077" begin
+    temp_pkg_dir() do project_path
+        Pkg.add("UUIDs")
+        # the following should not error
+        Pkg.add("UUIDs")
+        Pkg.test("UUIDs")
+        @test_throws PkgError("cannot `pin` stdlibs.") Pkg.pin("UUIDs")
+    end
+end
+
 #issue #975
 @testset "Pkg.gc" begin
     temp_pkg_dir() do project_path
         with_temp_env() do
             Pkg.add("Example")
             Pkg.gc()
+            # issue #601 and #1228
+            touch(joinpath(Pkg.depots1(), "packages", ".DS_Store"))
+            touch(joinpath(Pkg.depots1(), "packages", "Example", ".DS_Store"))
+            Pkg.gc()
         end
     end
 end
 
+#issue #876
+@testset "targets should survive add/rm" begin
+    temp_pkg_dir() do project_path; cd_tempdir() do tmpdir
+        cp(joinpath(@__DIR__, "project", "good", "pkg.toml"), "Project.toml")
+        targets = deepcopy(Pkg.Types.read_project("Project.toml").targets)
+        Pkg.activate(".")
+        Pkg.add("Example")
+        Pkg.rm("Example")
+        @test targets == Pkg.Types.read_project("Project.toml").targets
+    end end
+end
+
+@testset "reading corrupted project files" begin
+    dir = joinpath(@__DIR__, "project", "bad")
+    for bad_project in joinpath.(dir, readdir(dir))
+        @test_throws PkgError Pkg.Types.read_project(bad_project)
+    end
+end
+
+@testset "reading corrupted manifest files" begin
+    dir = joinpath(@__DIR__, "manifest", "bad")
+    for bad_manifest in joinpath.(dir, readdir(dir))
+        @test_throws PkgError Pkg.Types.read_manifest(bad_manifest)
+    end
+end
+
+@testset "Unregistered UUID in manifest" begin
+    temp_pkg_dir() do project_path; with_temp_env() do; cd_tempdir() do tmpdir
+        cp(joinpath(@__DIR__, "test_packages", "UnregisteredUUID"), "UnregisteredUUID")
+        Pkg.activate("UnregisteredUUID")
+        @test_throws PkgError Pkg.update()
+    end end end
+end
+
+@testset "canonicalized relative paths in manifest" begin
+    mktempdir() do tmp; cd(tmp) do
+        write("Manifest.toml",
+            """
+            [[Foo]]
+            path = "bar/Foo"
+            uuid = "824dc81a-29a7-11e9-3958-fba342a32644"
+            version = "0.1.0"
+            """)
+        manifest = Pkg.Types.read_manifest("Manifest.toml")
+        package = manifest[Base.UUID("824dc81a-29a7-11e9-3958-fba342a32644")]
+        @test package.path == (Sys.iswindows() ? "bar\\Foo" : "bar/Foo")
+        Pkg.Types.write_manifest(manifest, "Manifest.toml")
+        @test occursin("path = \"bar/Foo\"", read("Manifest.toml", String))
+    end end
+end
+
+@testset "building project should fix version of deps" begin
+    temp_pkg_dir() do project_path
+        dep_pkg = joinpath(@__DIR__, "test_packages", "BuildProjectFixedDeps")
+        Pkg.activate(dep_pkg)
+        Pkg.build()
+        @test isfile(joinpath(dep_pkg, "deps", "artifact"))
+    end
+end
+
+@testset "PkgError printing" begin
+    err = PkgError("foobar")
+    @test occursin("PkgError(\"foobar\")", sprint(show, err))
+    @test sprint(showerror, err) == "foobar"
+end
+
+@testset "issue #1066: package with colliding name/uuid exists in project" begin
+    temp_pkg_dir() do project_path; cd_tempdir() do tmpdir
+        Pkg.activate(".")
+        Pkg.generate("A")
+        cd(mkdir("packages")) do
+            Pkg.generate("A")
+            LibGit2.with(LibGit2.init("A")) do repo
+                LibGit2.add!(repo, "*")
+                LibGit2.commit(repo, "initial commit"; author=TEST_SIG, committer=TEST_SIG)
+            end
+        end
+        Pkg.generate("B")
+        project = Pkg.Types.read_project("A/Project.toml")
+        project.name = "B"
+        Pkg.Types.write_project(project, "B/Project.toml")
+        LibGit2.with(LibGit2.init("B")) do repo
+            LibGit2.add!(repo, "*")
+            LibGit2.commit(repo, "initial commit"; author=TEST_SIG, committer=TEST_SIG)
+        end
+        Pkg.develop(Pkg.PackageSpec(path = abspath("A")))
+        # package with same name but different uuid exist in project
+        @test_throws PkgError Pkg.develop(Pkg.PackageSpec(path = abspath("packages", "A")))
+        @test_throws PkgError Pkg.add(Pkg.PackageSpec(path = abspath("packages", "A")))
+        # package with same uuid but different name exist in project
+        @test_throws PkgError Pkg.develop(Pkg.PackageSpec(path = abspath("B")))
+        @test_throws PkgError Pkg.add(Pkg.PackageSpec(path = abspath("B")))
+    end end
+end
+
+@testset "issue #1180: broken toml-files in HEAD" begin
+    temp_pkg_dir() do dir; cd(dir) do
+        write("Project.toml", "[deps]\nExample = \n")
+        LibGit2.with(LibGit2.init(dir)) do repo
+            LibGit2.add!(repo, "*")
+            LibGit2.commit(repo, "initial commit"; author=TEST_SIG, committer=TEST_SIG)
+        end
+        write("Project.toml", "[deps]\nExample = \"7876af07-990d-54b4-ab0e-23690620f79a\"\n")
+        Pkg.activate(dir)
+        @test_logs (:warn, r"Could not read project from HEAD") Pkg.status()
+    end end
+end
+
+import Markdown
+@testset "REPL command doc generation" begin
+    # test that the way doc building extracts
+    # docstrings for Pkg REPL commands work
+    d = Dict(Pkg.REPLMode.canonical_names())
+    @test d["add"].help isa Markdown.MD
+    @test d["registry add"].help isa Markdown.MD
+end
+
+@testset "instantiate should respect tree hash" begin
+    temp_pkg_dir() do project_path; mktempdir() do tmp
+        copy_test_package(tmp, "NotUpdated")
+        Pkg.activate(joinpath(tmp, "NotUpdated"))
+        hash = Pkg.Types.Context().env.manifest[TEST_PKG.uuid].tree_hash
+        Pkg.instantiate()
+        @test hash == Pkg.Types.Context().env.manifest[TEST_PKG.uuid].tree_hash
+    end end
+end
+
+@testset "Issue 1124 code path" begin
+    temp_pkg_dir() do project_path; with_temp_env() do
+        pkg"add Example#master"
+        pkg"add Unicode"
+        pkg"up Unicode"
+    end end
+end
+
+@testset "instantiate checks for consistent dependency graph" begin
+    temp_pkg_dir() do project_path; mktempdir() do tmp
+        copy_test_package(tmp, "ExtraDirectDep")
+        Pkg.activate(joinpath(tmp, "ExtraDirectDep"))
+        @test_throws PkgError Pkg.instantiate()
+    end end
+end
+
+@testset "up should prune manifest" begin
+    example_uuid = UUID("7876af07-990d-54b4-ab0e-23690620f79a")
+    unicode_uuid = UUID("4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5")
+    temp_pkg_dir() do project_path; mktempdir() do tmp
+        copy_test_package(tmp, "Unpruned")
+        Pkg.activate(joinpath(tmp, "Unpruned"))
+        Pkg.update()
+        manifest = Pkg.Types.Context().env.manifest
+        package_example = get(manifest, example_uuid, nothing)
+        @test package_example !== nothing
+        @test package_example.version > v"0.4.0"
+        @test get(manifest, unicode_uuid, nothing) === nothing
+    end end
+end
+
 include("repl.jl")
 include("api.jl")
 include("registry.jl")
