@@ -172,7 +172,10 @@ let exename = `$(Base.julia_cmd()) --startup-file=no`
     # --procs
     @test readchomp(`$exename -q -p 2 -e "println(nworkers())"`) == "2"
     @test !success(`$exename -p 0`)
-    @test !success(`$exename --procs=1.0`)
+    let p = run(`$exename --procs=1.0`, wait=false)
+        wait(p)
+        @test p.exitcode == 1 && p.termsignal == 0
+    end
 
     # --machine-file
     # this does not check that machine file works,
@@ -327,7 +330,7 @@ let exename = `$(Base.julia_cmd()) --startup-file=no`
         @test !success(`$exename -E "$code" --depwarn=error`)
 
         @test readchomperrors(`$exename -E "$code" --depwarn=yes`) ==
-            (true, "true", "WARNING: Foo.Deprecated is deprecated, use NotDeprecated instead.\n  likely near no file:5")
+            (true, "true", "WARNING: Foo.Deprecated is deprecated, use NotDeprecated instead.\n  likely near none:8")
 
         @test readchomperrors(`$exename -E "$code" --depwarn=no`) ==
             (true, "true", "")
