@@ -71,10 +71,8 @@ tmerge_test(Tuple{}, Tuple{Complex, Vararg{Union{ComplexF32, ComplexF64}}},
     Tuple{Vararg{Complex}})
 @test Core.Compiler.tmerge(Tuple{}, Union{Int16, Nothing, Tuple{ComplexF32, ComplexF32}}) ==
     Union{Int16, Nothing, Tuple{Vararg{ComplexF32}}}
-@test Core.Compiler.tmerge(Int32, Union{Int16, Nothing, Tuple{ComplexF32, ComplexF32}}) ==
-    Union{Int16, Int32, Nothing, Tuple{ComplexF32, ComplexF32}}
-@test Core.Compiler.tmerge(Union{Int32, Nothing, Tuple{ComplexF32}}, Union{Int16, Nothing, Tuple{ComplexF32, ComplexF32}}) ==
-    Union{Int16, Int32, Nothing, Tuple{Vararg{ComplexF32}}}
+@test Core.Compiler.tmerge(Union{Int32, Nothing, Tuple{ComplexF32}}, Union{Int32, Nothing, Tuple{ComplexF32, ComplexF32}}) ==
+    Union{Int32, Nothing, Tuple{Vararg{ComplexF32}}}
 
 # issue 9770
 @noinline x9770() = false
@@ -613,6 +611,29 @@ mutable struct HasAbstractlyTypedField
 end
 f_infer_abstract_fieldtype() = fieldtype(HasAbstractlyTypedField, :x)
 @test Base.return_types(f_infer_abstract_fieldtype, ()) == Any[Type{Union{Int,String}}]
+let fieldtype_tfunc = Core.Compiler.fieldtype_tfunc,
+    fieldtype_nothrow = Core.Compiler.fieldtype_nothrow
+    @test fieldtype_tfunc(Union{}, :x) == Union{}
+    @test fieldtype_tfunc(Union{Type{Int32}, Int32}, Const(:x)) == Union{}
+    @test fieldtype_tfunc(Union{Type{Base.RefValue{T}}, Type{Int32}} where {T<:Array}, Const(:x)) == Type{<:Array}
+    @test fieldtype_tfunc(Union{Type{Base.RefValue{T}}, Type{Int32}} where {T<:Real}, Const(:x)) == Type{<:Real}
+    @test fieldtype_tfunc(Union{Type{Base.RefValue{<:Array}}, Type{Int32}}, Const(:x)) == Type{Array}
+    @test fieldtype_tfunc(Union{Type{Base.RefValue{<:Real}}, Type{Int32}}, Const(:x)) == Const(Real)
+    @test fieldtype_tfunc(Const(Union{Base.RefValue{<:Real}, Type{Int32}}), Const(:x)) == Type
+    @test fieldtype_tfunc(Type{Union{Base.RefValue{T}, Type{Int32}}} where {T<:Real}, Const(:x)) == Type
+    @test fieldtype_tfunc(Type{<:Tuple}, Const(1)) == Type
+    @test fieldtype_tfunc(Type{<:Tuple}, Any) == Type
+    @test fieldtype_nothrow(Type{Base.RefValue{<:Real}}, Const(:x))
+    @test !fieldtype_nothrow(Type{Union{}}, Const(:x))
+    @test !fieldtype_nothrow(Union{Type{Base.RefValue{T}}, Int32} where {T<:Real}, Const(:x))
+    @test !fieldtype_nothrow(Union{Type{Base.RefValue{<:Real}}, Int32}, Const(:x))
+    @test fieldtype_nothrow(Const(Union{Base.RefValue{<:Real}, Int32}), Const(:x))
+    @test !fieldtype_nothrow(Type{Union{Base.RefValue{T}, Int32}} where {T<:Real}, Const(:x)) # improvable?
+    @test fieldtype_nothrow(Union{Type{Base.RefValue{T}}, Type{Base.RefValue{Any}}} where {T<:Real}, Const(:x))
+    @test fieldtype_nothrow(Union{Type{Base.RefValue{<:Real}}, Type{Base.RefValue{Any}}}, Const(:x))
+    @test fieldtype_nothrow(Const(Union{Base.RefValue{<:Real}, Base.RefValue{Any}}), Const(:x))
+    @test fieldtype_nothrow(Type{Union{Base.RefValue{T}, Base.RefValue{Any}}} where {T<:Real}, Const(:x))
+end
 
 # issue #11480
 @noinline f11480(x,y) = x
@@ -739,7 +760,7 @@ f19957(::Int) = Int8(1)            # issue #19957, inference failure when splatt
 f19957(::Int...) = Int16(1)
 f19957(::Any...) = "no"
 g19957(x) = f19957(x...)
-@test all(t -> t<:Union{Int8,Int16}, Base.return_types(g19957, (Int,))) # with a full fix, this should just be Int8
+@test Base.return_types(g19957, (Int,)) == Any[Int8]
 
 # Inference for some type-level computation
 fUnionAll(::Type{T}) where {T} = Type{S} where S <: T
@@ -958,7 +979,7 @@ end
 # but which also means we should still be storing the inference result from inferring the cycle
 f21653() = f21653()
 @test code_typed(f21653, Tuple{}, optimize=false)[1] isa Pair{CodeInfo, typeof(Union{})}
-@test which(f21653, ()).specializations.func.rettype === Union{}
+@test which(f21653, ()).specializations.func.cache.rettype === Union{}
 
 # issue #22290
 f22290() = return 3
@@ -1052,13 +1073,13 @@ copy_dims_out(out) = ()
 copy_dims_out(out, dim::Int, tail...) =  copy_dims_out((out..., dim), tail...)
 copy_dims_out(out, dim::Colon, tail...) = copy_dims_out((out..., dim), tail...)
 @test Base.return_types(copy_dims_out, (Tuple{}, Vararg{Union{Int,Colon}})) == Any[Tuple{}, Tuple{}, Tuple{}]
-@test all(m -> 20 < count_specializations(m) < 45, methods(copy_dims_out))
+@test all(m -> 4 < count_specializations(m) < 15, methods(copy_dims_out)) # currently about 5
 
 copy_dims_pair(out) = ()
 copy_dims_pair(out, dim::Int, tail...) =  copy_dims_pair(out => dim, tail...)
 copy_dims_pair(out, dim::Colon, tail...) = copy_dims_pair(out => dim, tail...)
 @test Base.return_types(copy_dims_pair, (Tuple{}, Vararg{Union{Int,Colon}})) == Any[Tuple{}, Tuple{}, Tuple{}]
-@test all(m -> 10 < count_specializations(m) < 35, methods(copy_dims_pair))
+@test all(m -> 5 < count_specializations(m) < 15, methods(copy_dims_pair)) # currently about 7
 
 @test isdefined_tfunc(typeof(NamedTuple()), Const(0)) === Const(false)
 @test isdefined_tfunc(typeof(NamedTuple()), Const(1)) === Const(false)
@@ -1088,27 +1109,25 @@ function get_linfo(@nospecialize(f), @nospecialize(t))
         throw(ArgumentError("argument is not a generic function"))
     end
     # get the MethodInstance for the method match
-    world = typemax(UInt)
     meth = which(f, t)
     t = Base.to_tuple_type(t)
     ft = isa(f, Type) ? Type{f} : typeof(f)
     tt = Tuple{ft, t.parameters...}
-    precompile(tt)
+    precompile(tt) # does inference (calls jl_type_infer) on this signature
     (ti, env) = ccall(:jl_type_intersection_with_env, Ref{Core.SimpleVector}, (Any, Any), tt, meth.sig)
-    meth = Base.func_for_method_checked(meth, tt)
     return ccall(:jl_specializations_get_linfo, Ref{Core.MethodInstance},
-                 (Any, Any, Any, UInt), meth, tt, env, world)
+                 (Any, Any, Any), meth, tt, env)
 end
 
 function test_const_return(@nospecialize(f), @nospecialize(t), @nospecialize(val))
-    linfo = get_linfo(f, t)
+    linfo = Core.Compiler.inf_for_methodinstance(get_linfo(f, t), Core.Compiler.get_world_counter())::Core.CodeInstance
     # If coverage is not enabled, make the check strict by requiring constant ABI
     # Otherwise, check the typed AST to make sure we return a constant.
     if Base.JLOptions().code_coverage == 0
         @test Core.Compiler.invoke_api(linfo) == 2
     end
     if Core.Compiler.invoke_api(linfo) == 2
-        @test linfo.inferred_const == val
+        @test linfo.rettype_const == val
         return
     end
     ct = code_typed(f, t)
@@ -1165,8 +1184,8 @@ test_const_return(()->sizeof(1), Tuple{}, sizeof(Int))
 test_const_return(()->sizeof(DataType), Tuple{}, sizeof(DataType))
 test_const_return(()->sizeof(1 < 2), Tuple{}, 1)
 test_const_return(()->fieldtype(Dict{Int64,Nothing}, :age), Tuple{}, UInt)
-@eval test_const_return(()->Core.sizeof($(Array{Int,0}(undef))), Tuple{}, sizeof(Int))
-@eval test_const_return(()->Core.sizeof($(Matrix{Float32}(undef, 2, 2))), Tuple{}, 4 * 2 * 2)
+test_const_return(@eval(()->Core.sizeof($(Array{Int,0}(undef)))), Tuple{}, sizeof(Int))
+test_const_return(@eval(()->Core.sizeof($(Matrix{Float32}(undef, 2, 2)))), Tuple{}, 4 * 2 * 2)
 
 # Make sure Core.sizeof with a ::DataType as inferred input type is inferred but not constant.
 function sizeof_typeref(typeref)
@@ -1340,6 +1359,26 @@ let egal_tfunc
     @test egal_tfunc(Union{Int64, Float64}, AbstractArray) === Const(false)
 end
 
+using Core.Compiler: PartialStruct, nfields_tfunc, sizeof_tfunc, sizeof_nothrow
+let PT = PartialStruct(Tuple{Int64,UInt64}, Any[Const(10, false), UInt64])
+    @test sizeof_tfunc(PT) === Const(16, false)
+    @test nfields_tfunc(PT) === Const(2, false)
+    @test sizeof_nothrow(PT) === true
+end
+@test sizeof_nothrow(Const(Tuple)) === false
+
+using Core.Compiler: typeof_tfunc
+@test typeof_tfunc(Tuple{Vararg{Int}}) == Type{Tuple{Vararg{Int,N}}} where N
+@test typeof_tfunc(Tuple{Any}) == Type{<:Tuple{Any}}
+@test typeof_tfunc(Type{Array}) === DataType
+@test typeof_tfunc(Type{<:Array}) === DataType
+@test typeof_tfunc(Array{Int}) == Type{Array{Int,N}} where N
+@test typeof_tfunc(AbstractArray{Int}) == Type{<:AbstractArray{Int,N}} where N
+@test typeof_tfunc(Union{<:T, <:Real} where T<:Complex) == Union{Type{Complex{T}} where T<:Real, Type{<:Real}}
+
+f_typeof_tfunc(x) = typeof(x)
+@test Base.return_types(f_typeof_tfunc, (Union{<:T, Int} where T<:Complex,)) == Any[Union{Type{Int}, Type{Complex{T}} where T<:Real}]
+
 function f23024(::Type{T}, ::Int) where T
     1 + 1
 end
@@ -1374,16 +1413,16 @@ gg13183(x::X...) where {X} = (_false13183 ? gg13183(x, x) : 0)
 
 # test the external OptimizationState constructor
 let linfo = get_linfo(Base.convert, Tuple{Type{Int64}, Int32}),
-    world = typemax(UInt),
+    world = UInt(23) # some small-numbered world that should be valid
     opt = Core.Compiler.OptimizationState(linfo, Core.Compiler.Params(world))
     # make sure the state of the properties look reasonable
     @test opt.src !== linfo.def.source
-    @test length(opt.src.slotflags) == length(opt.src.slotnames)
+    @test length(opt.src.slotflags) == linfo.def.nargs <= length(opt.src.slotnames)
     @test opt.src.ssavaluetypes isa Vector{Any}
     @test !opt.src.inferred
     @test opt.mod === Base
-    @test opt.max_valid === typemax(UInt)
-    @test opt.min_valid === Core.Compiler.min_world(opt.linfo) > 2
+    @test opt.max_valid === Core.Compiler.get_world_counter()
+    @test opt.min_valid === Core.Compiler.min_world(opt.src) === UInt(1)
     @test opt.nargs == 3
 end
 
@@ -1841,6 +1880,15 @@ function inbounds_30563()
 end
 @test Base.return_types(inbounds_30563, ()) == Any[Int]
 
+function ifs_around_var_capture()
+    if false end
+    x = 1
+    if false end
+    f = y->x
+    f(0)
+end
+@test Base.return_types(ifs_around_var_capture, ()) == Any[Int]
+
 # issue #27316 - inference shouldn't hang on these
 f27316(::Vector) = nothing
 f27316(::Any) = f27316(Any[][1]), f27316(Any[][1])
@@ -1880,6 +1928,12 @@ Base.iterate(::Iterator27434, ::Any) = nothing
 @test @inferred((1, 2, 3) == (1, 2, 3))
 @test Core.Compiler.return_type(splat27434, Tuple{typeof(Iterators.repeated(1))}) == Union{}
 
+# issue #32465
+let rt = Base.return_types(splat27434, (NamedTuple{(:x,), Tuple{T}} where T,))
+    @test rt == Any[Tuple{Any}]
+    @test !Base.has_free_typevars(rt[1])
+end
+
 # issue #27078
 f27078(T::Type{S}) where {S} = isa(T, UnionAll) ? f27078(T.body) : T
 T27078 = Vector{Vector{T}} where T
@@ -2193,3 +2247,177 @@ j30385(T, y) = k30385(f30385(T, y))
 
 @test @inferred(j30385(AbstractFloat, 1)) == 1
 @test @inferred(j30385(:dummy, 1)) == "dummy"
+
+@test Base.return_types(Tuple, (NamedTuple{<:Any,Tuple{Any,Int}},)) == Any[Tuple{Any,Int}]
+@test Base.return_types(Base.splat(tuple), (typeof((a=1,)),)) == Any[Tuple{Int}]
+
+# test that return_type_tfunc isn't affected by max_methods differently than return_type
+_rttf_test(::Int8) = 0
+_rttf_test(::Int16) = 0
+_rttf_test(::Int32) = 0
+_rttf_test(::Int64) = 0
+_rttf_test(::Int128) = 0
+_call_rttf_test() = Core.Compiler.return_type(_rttf_test, Tuple{Any})
+@test Core.Compiler.return_type(_rttf_test, Tuple{Any}) === Int
+@test _call_rttf_test() === Int
+
+f_with_Type_arg(::Type{T}) where {T} = T
+@test Base.return_types(f_with_Type_arg, (Any,)) == Any[Type]
+@test Base.return_types(f_with_Type_arg, (Type{Vector{T}} where T,)) == Any[Type{Vector{T}} where T]
+
+# Generated functions that only reference some of their arguments
+@inline function my_ntuple(f::F, ::Val{N}) where {F,N}
+    N::Int
+    (N >= 0) || throw(ArgumentError(string("tuple length should be ≥0, got ", N)))
+    if @generated
+        quote
+            @Base.nexprs $N i -> t_i = f(i)
+            @Base.ncall $N tuple t
+        end
+    else
+        Tuple(f(i) for i = 1:N)
+    end
+end
+call_ntuple(a, b) = my_ntuple(i->(a+b; i), Val(4))
+@test Base.return_types(call_ntuple, Tuple{Any,Any}) == [NTuple{4, Int}]
+@test length(code_typed(my_ntuple, Tuple{Any, Val{4}})) == 1
+@test_throws ErrorException code_typed(my_ntuple, Tuple{Any, Val})
+
+@generated unionall_sig_generated(::Vector{T}, b::Vector{S}) where {T, S} = :($b)
+@test length(code_typed(unionall_sig_generated, Tuple{Any, Vector{Int}})) == 1
+
+# Test that we don't limit recursions on the number of arguments, even if the
+# arguments themselves are getting more complex
+f_incr(x::Tuple, y::Tuple, args...) = f_incr((x, y), args...)
+f_incr(x::Tuple) = x
+@test @inferred(f_incr((), (), (), (), (), (), (), ())) ==
+    ((((((((), ()), ()), ()), ()), ()), ()), ())
+
+# Test PartialStruct for closures
+@noinline use30783(x) = nothing
+function foo30783(b)
+    a = 1
+    f = ()->(use30783(b); Val(a))
+    f()
+end
+@test @inferred(foo30783(2)) == Val(1)
+
+# PartialStruct tmerge
+using Core.Compiler: PartialStruct, tmerge, Const, ⊑
+struct FooPartial
+    a::Int
+    b::Int
+    c::Int
+end
+let PT1 = PartialStruct(FooPartial, Any[Const(1), Const(2), Int]),
+    PT2 = PartialStruct(FooPartial, Any[Const(1), Int, Int]),
+    PT3 = PartialStruct(FooPartial, Any[Const(1), Int, Const(3)])
+
+    @test PT1 ⊑ PT2
+    @test !(PT1 ⊑ PT3) && !(PT2 ⊑ PT1)
+    let (==) = (a, b)->(a ⊑ b && b ⊑ a)
+        @test tmerge(PT1, PT3) == PT2
+    end
+end
+
+# issue 31164
+struct NoInit31164
+    a::Int
+    b::Any
+    NoInit31164(a::Int) = new(a)
+    NoInit31164(a::Int, b) = new(a, b)
+end
+
+@eval function foo31164(b, x)
+    if b
+       a = NoInit31164(1, x)
+    else
+       a = $(NoInit31164(1))
+    end
+    return a
+end
+
+@test_nowarn code_typed(foo31164, Tuple{Bool, Int}; optimize=false)
+
+# there are errors when these functions are defined inside the @testset
+f28762(::Type{<:AbstractArray{T}}) where {T} = T
+f28762(::Type{<:AbstractArray}) = Any
+g28762(::Type{X}) where {X} = Array{eltype(X)}(undef, 0)
+h28762(::Type{X}) where {X} = Array{f28762(X)}(undef, 0)
+
+@testset "@inferred bug from #28762" begin
+    # this works since Julia 1.1
+    @test (@inferred eltype(Array)) == Any
+    @test (@inferred f28762(Array)) == Any
+    @inferred g28762(Array{Int})
+    @inferred h28762(Array{Int})
+    @inferred g28762(Array)
+    @inferred h28762(Array)
+end
+
+# issue #31663
+module I31663
+abstract type AbstractNode end
+
+struct Node{N1<:AbstractNode, N2<:AbstractNode} <: AbstractNode
+    a::N1
+    b::N2
+end
+
+struct Leaf <: AbstractNode
+end
+
+function gen_nodes(qty::Integer) :: AbstractNode
+    @assert qty > 0
+    result = Leaf()
+    for i in 1:qty
+        result = Node(result, Leaf())
+    end
+    return result
+end
+end
+@test count(==('}'), string(I31663.gen_nodes(50))) == 1275
+
+# issue #31572
+struct MixedKeyDict{T<:Tuple} #<: AbstractDict{Any,Any}
+    dicts::T
+end
+Base.merge(f::Function, d::MixedKeyDict, others::MixedKeyDict...) = _merge(f, (), d.dicts, (d->d.dicts).(others)...)
+Base.merge(f, d::MixedKeyDict, others::MixedKeyDict...) = _merge(f, (), d.dicts, (d->d.dicts).(others)...)
+function _merge(f, res, d, others...)
+    ofsametype, remaining = _alloftype(Base.heads(d), ((),), others...)
+    return _merge(f, (res..., merge(f, ofsametype...)), Base.tail(d), remaining...)
+end
+_merge(f, res, ::Tuple{}, others...) = _merge(f, res, others...)
+_merge(f, res, d) = MixedKeyDict((res..., d...))
+_merge(f, res, ::Tuple{}) = MixedKeyDict(res)
+function _alloftype(ofdesiredtype::Tuple{Vararg{D}}, accumulated, d::Tuple{D,Vararg}, others...) where D
+    return _alloftype((ofdesiredtype..., first(d)),
+                      (Base.front(accumulated)..., (last(accumulated)..., Base.tail(d)...), ()),
+                      others...)
+end
+function _alloftype(ofdesiredtype, accumulated, d, others...)
+    return _alloftype(ofdesiredtype,
+                      (Base.front(accumulated)..., (last(accumulated)..., first(d))),
+                      Base.tail(d), others...)
+end
+function _alloftype(ofdesiredtype, accumulated, ::Tuple{}, others...)
+    return _alloftype(ofdesiredtype,
+                      (accumulated..., ()),
+                      others...)
+end
+_alloftype(ofdesiredtype, accumulated) = ofdesiredtype, Base.front(accumulated)
+let
+    d = MixedKeyDict((Dict(1 => 3), Dict(4. => 2)))
+    e = MixedKeyDict((Dict(1 => 7), Dict(5. => 9)))
+    @test merge(+, d, e).dicts == (Dict(1 => 10), Dict(4.0 => 2, 5.0 => 9))
+    f = MixedKeyDict((Dict(2 => 7), Dict(5. => 11)))
+    @test merge(+, d, e, f).dicts == (Dict(1 => 10, 2 => 7), Dict(4.0 => 2, 5.0 => 20))
+end
+
+# Issue #31974
+f31974(a::UnitRange) = (if first(a) <= last(a); f31974((first(a)+1):last(a)); end; a)
+f31974(n::Int) = f31974(1:n)
+# This query hangs if type inference improperly attempts to const prop
+# call cycles.
+@test code_typed(f31974, Tuple{Int}) !== nothing
